//--------------------------------------------------------------------------------------
// File: main.cpp
//
// Desc: Syndicate Wars Vehicle Reader test file
//
// Written by Marco Castrucci <mmacleod@tiscali.it> in 2006
//
// Notes: The program requires thhe following files
//        .\data\PRIMVEH.OBJ
//        .\data\TEX00.DAT
//        .\data\TEX01.DAT
//        .\data\TEX02.DAT
//        .\data\TEX03.DAT
//        .\data\TEX04.DAT
//        .\data\TEX05.DAT
//        .\data\PAL.PAL
//--------------------------------------------------------------------------------------

#include "dxstdafx.h"
#include "resource.h"
#include "sw_vehrd.h"
#include <assert.h>
#include <stdlib.h>
#include <stdio.h>
#include <signal.h>


//--------------------------------------------------------------------------------------
// Forward declarations 
//--------------------------------------------------------------------------------------
void    InitApp();
void    RenderText();
void    CALLBACK KeyboardProc( UINT nChar, bool bKeyDown, bool bAltDown, void* pUserContext );
void    CALLBACK OnGUIEvent( UINT nEvent, int nControlID, CDXUTControl* pControl, void* pUserContext );



//--------------------------------------------------------------------------------------
// Global variables
//--------------------------------------------------------------------------------------
CVehiclesReader *g_vr = NULL;

CModelViewerCamera      g_Camera;               // A model viewing camera
IDirect3DVertexBuffer9	*g_vbuf = NULL;
LPDIRECT3DTEXTURE9		g_texture = NULL;
ID3DXFont*              g_pFont = NULL;            // Font for drawing text
ID3DXSprite*            g_pTextSprite = NULL;      // Sprite for batching draw text calls
CDXUTDialog             g_SampleUI;             // dialog for sample specific controls
CDXUTDialogResourceManager g_DialogResourceManager; // manager for shared resources of dialogs
bool                    g_bShowHelp = true;        // If true, it renders the UI control text
unsigned int			g_numVertices = 0;
unsigned int			g_vehicleIndex = 0;
unsigned int			g_requestedVehicleIndex = 0xFFFF;
int                     g_nShowMode = 0;	//0:all, 1:tri, 2:quad


//--------------------------------------------------------------------------------------
// UI control IDs
//--------------------------------------------------------------------------------------
#define IDC_VEHICLE_NUMBER      1
#define IDC_SCROLLLEFT          2
#define IDC_SCROLLRIGHT         3
#define IDC_SHOW_ALL			4
#define IDC_SHOW_ONLY_TRI		5
#define IDC_SHOW_ONLY_QUAD		6


struct CUSTOMVERTEX
{
    FLOAT x,y,z;
	FLOAT u,v;
};

#define D3DFVF_CUSTOMVERTEX (D3DFVF_XYZ|D3DFVF_TEX1)

#define WND_NAME	L"Syndicate Wars Vehicles Reader v0.01"

//--------------------------------------------------------------------------------------
// Error Message
//--------------------------------------------------------------------------------------
void ErrorMessage(const char *fmt,...)
{
	char message[1024];
	va_list list;

	va_start(list,fmt);
	_vsnprintf_s(message,1024,1024,fmt,list);
	va_end(list);

	wchar_t unimsg[1024*2];
	MultiByteToWideChar(CP_ACP, 0, message, -1, unimsg, 1024);

	switch( MessageBox(NULL,unimsg,TEXT("ERROR"),MB_ABORTRETRYIGNORE|MB_ICONERROR|MB_TASKMODAL|MB_SETFOREGROUND) )
	{
		case IDABORT: raise(SIGABRT); break;
		case IDCANCEL: break;
		case IDRETRY: __asm { int 3 }; break;
	}
}


//--------------------------------------------------------------------------------------
// Upload texture chunk
//--------------------------------------------------------------------------------------
void UploadTexture(unsigned int _xoffset,unsigned int _yoffset,void *_dst,unsigned int _pitch,const char *_imgPath,const char *_palPath)
{
	struct SRGB
	{
		BYTE b,g,r;
	};
	
	BYTE raw[256*256];
	SRGB pal[256];

	FILE *fp;

	fopen_s(&fp,_imgPath,"rb");	assert(fp);
	fread(raw,sizeof(BYTE),256*256,fp);
	fclose(fp);

	fopen_s(&fp,_palPath,"rb");	assert(fp);
	fread(pal,sizeof(SRGB),256,fp);
	fclose(fp);

	BYTE *src = raw;
	for(unsigned int y=0; y<256; y++)
	{
		BYTE *dst = (BYTE*)_dst + (_yoffset+y)*_pitch + _xoffset*4;	//4 is texture pixel size
		
		for(unsigned int x=0; x<256; x++)
		{
			dst[0] = pal[*src].r * 4;
			dst[1] = pal[*src].g * 4;
			dst[2] = pal[*src].b * 4;
			dst[3] = 0;
			dst += 4;
			src++;
		}
	}
	
}

//--------------------------------------------------------------------------------------
// Create the main texture
//--------------------------------------------------------------------------------------
void CreateTexture(IDirect3DDevice9* pd3dDevice)
{
	HRESULT hr;

	V( D3DXCreateTexture( pd3dDevice, 256*4, 256*2, 0, 0, D3DFMT_X8R8G8B8, D3DPOOL_MANAGED, &g_texture) );

	LPDIRECT3DSURFACE9 pTxtSurface;
	V( g_texture->GetSurfaceLevel( 0, &pTxtSurface) );
	
	D3DLOCKED_RECT rc;
	V( pTxtSurface->LockRect( &rc, NULL, 0) );

	UploadTexture(256*0,256*0,rc.pBits,rc.Pitch,"./data/tex00.dat","./data/pal.pal");
	UploadTexture(256*1,256*0,rc.pBits,rc.Pitch,"./data/tex01.dat","./data/pal.pal");
	UploadTexture(256*2,256*0,rc.pBits,rc.Pitch,"./data/tex02.dat","./data/pal.pal");
	UploadTexture(256*3,256*0,rc.pBits,rc.Pitch,"./data/tex03.dat","./data/pal.pal");
	UploadTexture(256*0,256*1,rc.pBits,rc.Pitch,"./data/tex04.dat","./data/pal.pal");
	UploadTexture(256*1,256*1,rc.pBits,rc.Pitch,"./data/tex05.dat","./data/pal.pal");

	V( pTxtSurface->UnlockRect() );
	pTxtSurface->Release();

//	D3DXSaveTextureToFile(TEXT("c:/ciao.bmp"),D3DXIFF_BMP,g_texture,NULL);	//debug output
}

void SetupVehicleData(int _vehicleIndex,IDirect3DDevice9* pd3dDevice)
{
	HRESULT hr;

	SAFE_RELEASE(g_vbuf);

	if(g_texture == NULL)
		CreateTexture(pd3dDevice);

	CVehiclesReader::SMesh *mesh = &g_vr->m_meshes[_vehicleIndex];

	{//upload vertices
		CUSTOMVERTEX *vert;
		
		g_numVertices = 0;
		if(g_nShowMode == 0)
			g_numVertices = (mesh->numTriFaces + mesh->numQuadFaces * 2) * 3;
		else if(g_nShowMode == 1)
			g_numVertices = (mesh->numTriFaces) * 3;
		else
			g_numVertices = (mesh->numQuadFaces * 2) * 3;

		if(!g_numVertices)
			return;

		V( pd3dDevice->CreateVertexBuffer( sizeof(CUSTOMVERTEX)*g_numVertices, 0, D3DFVF_CUSTOMVERTEX,
											D3DPOOL_MANAGED, &g_vbuf, NULL) );

		V( g_vbuf->Lock( 0, 0, (void**)&vert, 0 ) );

		u32 minVertex = 0xFFFFFFFF;	// just for debugging purpouse
		u32 maxVertex = 0;

		if(mesh->numTriFaces && (g_nShowMode != 2))		// upload tri faces
		{
			u16 idx[3];	// vertex index
			float uv[4][2];	//texcoords values

			u32 faceIndex = mesh->firstTriFace;

			for(u32 i=0; i<mesh->numTriFaces; i++,faceIndex++)
			{
				idx[0] = g_vr->m_triFaces[faceIndex].v0;
				idx[1] = g_vr->m_triFaces[faceIndex].v1;
				idx[2] = g_vr->m_triFaces[faceIndex].v2;

				for(int k=0; k<3; k++)	{if(idx[k] > maxVertex) maxVertex = idx[k];		if(idx[k] < minVertex) minVertex = idx[k];}

				u16 uvFaceIndex = g_vr->m_triFaces[faceIndex].uvFace;
				BYTE texNum = (BYTE)g_vr->m_triUVFaces[uvFaceIndex].texture;
				float u0 = (1.f/4.f)*(float)g_vr->m_triUVFaces[uvFaceIndex].u0 / 255.f;	// (1.f/4.f) because our texture is composed (width) by 4 syndicate wars textures 
				float v0 = (1.f/2.f)*(float)g_vr->m_triUVFaces[uvFaceIndex].v0 / 255.f;	// (1.f/2.f) because our texture is composed (height) by 2 syndicate wars textures 
				float u1 = (1.f/4.f)*(float)g_vr->m_triUVFaces[uvFaceIndex].u1 / 255.f;
				float v1 = (1.f/2.f)*(float)g_vr->m_triUVFaces[uvFaceIndex].v1 / 255.f;
				float u2 = (1.f/4.f)*(float)g_vr->m_triUVFaces[uvFaceIndex].u2 / 255.f;
				float v2 = (1.f/2.f)*(float)g_vr->m_triUVFaces[uvFaceIndex].v2 / 255.f;
			
				float uoffs = 0.f;
				float voffs = 0.f;
				switch(texNum)
				{
					case 0:
					case 1:
					case 2:
					case 3: uoffs = (float)texNum * (1.f/4.f);	break;
					case 4:
					case 5:
					case 6:
					case 7: uoffs = (float)texNum * (1.f/4.f);	voffs = (1.f/2.f);	break;
				}

				uv[0][0] = u0 + uoffs;	uv[0][1] = v0 + voffs;
				uv[1][0] = u1 + uoffs;	uv[1][1] = v1 + voffs;
				uv[2][0] = u2 + uoffs;	uv[2][1] = v2 + voffs;

				u16 idxMap[3] = {2,1,0};
				for(int j=0; j<3; j++, vert++)
				{
					vert->x = (float)g_vr->m_vertices[ idx[ idxMap[j] ] ].x;
					vert->y = (float)g_vr->m_vertices[ idx[ idxMap[j] ] ].y;
					vert->z = (float)g_vr->m_vertices[ idx[ idxMap[j] ] ].z;
					vert->u = uv[ idxMap[j] ][0];
					vert->v = uv[ idxMap[j] ][1];
				}
			}//..i.
		}//..tri.faces.


		if(mesh->numQuadFaces && (g_nShowMode != 1))	// upload quad faces
		{
			u16 idx[4];	//vertex index
			float uv[4][2];	//texcoords values

			int faceIndex = mesh->firstQuadFace;

			for(int i=0; i<mesh->numQuadFaces; i++, faceIndex++)
			{
				idx[0] = g_vr->m_quadFaces[faceIndex].v0;
				idx[1] = g_vr->m_quadFaces[faceIndex].v1;
				idx[2] = g_vr->m_quadFaces[faceIndex].v2;
				idx[3] = g_vr->m_quadFaces[faceIndex].v3;

				for(int k=0; k<4; k++)	{if(idx[k] > maxVertex) maxVertex = idx[k];		if(idx[k] < minVertex) minVertex = idx[k];}

				u16 uvFaceIndex = g_vr->m_quadFaces[faceIndex].uvFace;
				BYTE texNum = (BYTE)g_vr->m_quadUVFaces[uvFaceIndex].texture;
				float u0 = (1.f/4.f)*(float)g_vr->m_quadUVFaces[uvFaceIndex].u0 / 255.f;	// (1.f/4.f) because our texture is composed (width) by 4 syndicate wars textures 
				float v0 = (1.f/2.f)*(float)g_vr->m_quadUVFaces[uvFaceIndex].v0 / 255.f;	// (1.f/2.f) because our texture is composed (height) by 2 syndicate wars textures 
				float u1 = (1.f/4.f)*(float)g_vr->m_quadUVFaces[uvFaceIndex].u1 / 255.f;
				float v1 = (1.f/2.f)*(float)g_vr->m_quadUVFaces[uvFaceIndex].v1 / 255.f;
				float u2 = (1.f/4.f)*(float)g_vr->m_quadUVFaces[uvFaceIndex].u2 / 255.f;
				float v2 = (1.f/2.f)*(float)g_vr->m_quadUVFaces[uvFaceIndex].v2 / 255.f;
				float u3 = (1.f/4.f)*(float)g_vr->m_quadUVFaces[uvFaceIndex].u3 / 255.f;
				float v3 = (1.f/2.f)*(float)g_vr->m_quadUVFaces[uvFaceIndex].v3 / 255.f;
			
				float uoffs = 0.f;
				float voffs = 0.f;
				switch(texNum)
				{
					case 0:
					case 1:
					case 2:
					case 3: uoffs = (float)texNum * (1.f/4.f);	break;
					case 4:
					case 5:
					case 6:
					case 7: uoffs = (float)texNum * (1.f/4.f);	voffs = (1.f/2.f);	break;
				}

				uv[0][0] = u0 + uoffs;	uv[0][1] = v0 + voffs;
				uv[1][0] = u1 + uoffs;	uv[1][1] = v1 + voffs;
				uv[2][0] = u2 + uoffs;	uv[2][1] = v2 + voffs;
				uv[3][0] = u3 + uoffs;	uv[3][1] = v3 + voffs;

				u16 idxMap[6] = {2,1,0,2,3,1};
				for(int j=0; j<6; j++, vert++)
				{
					vert->x = (float)g_vr->m_vertices[ idx[ idxMap[j] ] ].x;
					vert->y = (float)g_vr->m_vertices[ idx[ idxMap[j] ] ].y;
					vert->z = (float)g_vr->m_vertices[ idx[ idxMap[j] ] ].z;
					vert->u = uv[ idxMap[j] ][0];
					vert->v = uv[ idxMap[j] ][1];
				}
			}//..i.
		}//..quad.faces.

		V( g_vbuf->Unlock() );
	}//..v.
}

//--------------------------------------------------------------------------------------
// Rejects any devices that aren't acceptable by returning false
//--------------------------------------------------------------------------------------
bool CALLBACK IsDeviceAcceptable( D3DCAPS9* pCaps, D3DFORMAT AdapterFormat, 
                                  D3DFORMAT BackBufferFormat, bool bWindowed, void* pUserContext )
{
    // Typically want to skip backbuffer formats that don't support alpha blending
    IDirect3D9* pD3D = DXUTGetD3DObject(); 
    if( FAILED( pD3D->CheckDeviceFormat( pCaps->AdapterOrdinal, pCaps->DeviceType,
                    AdapterFormat, D3DUSAGE_QUERY_POSTPIXELSHADER_BLENDING, 
                    D3DRTYPE_TEXTURE, BackBufferFormat ) ) )
        return false;

    return true;
}


//--------------------------------------------------------------------------------------
// Before a device is created, modify the device settings as needed
//--------------------------------------------------------------------------------------
bool CALLBACK ModifyDeviceSettings( DXUTDeviceSettings* pDeviceSettings, const D3DCAPS9* pCaps, void* pUserContext )
{
    return true;
}


//--------------------------------------------------------------------------------------
// Create any D3DPOOL_MANAGED resources here 
//--------------------------------------------------------------------------------------
HRESULT CALLBACK OnCreateDevice( IDirect3DDevice9* pd3dDevice, const D3DSURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext )
{
	HRESULT hr;

    V_RETURN( g_DialogResourceManager.OnCreateDevice( pd3dDevice ) );

    // Initialize the font
    V_RETURN( D3DXCreateFont( pd3dDevice, 15, 0, FW_BOLD, 1, FALSE, DEFAULT_CHARSET, 
                         OUT_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, 
                         L"Arial", &g_pFont ) );

    // Setup the camera's view parameters
    D3DXVECTOR3 vecEye(0.0f, 0.0f, 1200.0f);
    D3DXVECTOR3 vecAt (0.0f, 0.0f, 0.0f);
    g_Camera.SetViewParams( &vecEye, &vecAt );

	// setup vehicle
	SetupVehicleData(g_vehicleIndex, pd3dDevice);

    return S_OK;
}


//--------------------------------------------------------------------------------------
// Create any D3DPOOL_DEFAULT resources here 
//--------------------------------------------------------------------------------------
HRESULT CALLBACK OnResetDevice( IDirect3DDevice9* pd3dDevice, 
                                const D3DSURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext )
{
	HRESULT hr;

	V_RETURN( g_DialogResourceManager.OnResetDevice() );

    // Setup the camera's projection parameters
    float fAspectRatio = pBackBufferSurfaceDesc->Width / (FLOAT)pBackBufferSurfaceDesc->Height;
    g_Camera.SetProjParams( D3DX_PI/4, fAspectRatio, 2.0f, 9000.0f );
    g_Camera.SetWindow( pBackBufferSurfaceDesc->Width, pBackBufferSurfaceDesc->Height );
    g_Camera.SetButtonMasks( MOUSE_LEFT_BUTTON, MOUSE_WHEEL, MOUSE_MIDDLE_BUTTON );

    if( g_pFont )
        V_RETURN( g_pFont->OnResetDevice() );

    // Create a sprite to help batch calls when drawing many lines of text
    V_RETURN( D3DXCreateSprite( pd3dDevice, &g_pTextSprite ) );

    g_SampleUI.SetLocation( ( pBackBufferSurfaceDesc->Width-170 ) / 2, pBackBufferSurfaceDesc->Height-120 );
    g_SampleUI.SetSize( 170, 120 );

    return S_OK;
}


//--------------------------------------------------------------------------------------
// Handle updates to the scene
//--------------------------------------------------------------------------------------
void CALLBACK OnFrameMove( IDirect3DDevice9* pd3dDevice, double fTime, float fElapsedTime, void* pUserContext )
{
    // Update the camera's position based on user input 
    g_Camera.FrameMove( fElapsedTime );
}


//--------------------------------------------------------------------------------------
// Render the scene 
//--------------------------------------------------------------------------------------
void CALLBACK OnFrameRender( IDirect3DDevice9* pd3dDevice, double fTime, float fElapsedTime, void* pUserContext )
{
	if(g_requestedVehicleIndex != 0xFFFF)
	{
		g_vehicleIndex = g_requestedVehicleIndex;
		SetupVehicleData(g_vehicleIndex, pd3dDevice);
		g_requestedVehicleIndex = 0xFFFF;
	}

    HRESULT hr;

    // Clear the render target and the zbuffer 
    V( pd3dDevice->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, D3DCOLOR_ARGB(0, 45, 50, 170), 1.0f, 0) );

    // Render the scene
    if( SUCCEEDED( pd3dDevice->BeginScene() ) )
    {
		V( pd3dDevice->SetTransform(D3DTS_WORLD, g_Camera.GetWorldMatrix()) );
		V( pd3dDevice->SetTransform(D3DTS_VIEW, g_Camera.GetViewMatrix()) );
		V( pd3dDevice->SetTransform(D3DTS_PROJECTION, g_Camera.GetProjMatrix()) );
		V( pd3dDevice->SetRenderState( D3DRS_LIGHTING, FALSE ) );
		V( pd3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_NONE ) );

		V( pd3dDevice->SetTexture( 0, g_texture ) );
		V( pd3dDevice->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_MODULATE ) );
		V( pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE ) );
		V( pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE ) );
		V( pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAOP,   D3DTOP_DISABLE ) );

		V( pd3dDevice->SetStreamSource( 0, g_vbuf, 0, sizeof(CUSTOMVERTEX) ) );
		V( pd3dDevice->SetFVF( D3DFVF_CUSTOMVERTEX ) );

		V( pd3dDevice->DrawPrimitive( D3DPT_TRIANGLELIST, 0, g_numVertices/3) );

        // Render stats and help text
        RenderText();

		V( g_SampleUI.OnRender( fElapsedTime ) );

        V( pd3dDevice->EndScene() );
    }
}


//--------------------------------------------------------------------------------------
// Render the help and statistics text. This function uses the ID3DXFont interface for 
// efficient text rendering.
//--------------------------------------------------------------------------------------
void RenderText()
{
    // The helper object simply helps keep track of text position, and color
    // and then it calls pFont->DrawText( m_pSprite, strMsg, -1, &rc, DT_NOCLIP, m_clr );
    // If NULL is passed in as the sprite object, then it will work however the 
    // pFont->DrawText() will not be batched together.  Batching calls will improves performance.
    CDXUTTextHelper txtHelper( g_pFont, g_pTextSprite, 15 );

    // Output statistics
    txtHelper.Begin();
    txtHelper.SetInsertionPos( 5, 5 );
    txtHelper.SetForegroundColor( D3DXCOLOR( 1.0f, 1.0f, 0.0f, 1.0f ) );
    txtHelper.DrawTextLine( DXUTGetFrameStats() );
    txtHelper.DrawTextLine( DXUTGetDeviceStats() );

    txtHelper.SetForegroundColor( D3DXCOLOR( 1.0f, 1.0f, 1.0f, 1.0f ) );
    
    // Draw help
    const D3DSURFACE_DESC* pd3dsdBackBuffer = DXUTGetBackBufferSurfaceDesc();
    if( g_bShowHelp )
    {
        txtHelper.SetInsertionPos( 10, pd3dsdBackBuffer->Height-15*6 );
        txtHelper.SetForegroundColor( D3DXCOLOR( 1.0f, 0.75f, 0.0f, 1.0f ) );
        txtHelper.DrawTextLine( L"Controls:" );

        txtHelper.SetInsertionPos( 40, pd3dsdBackBuffer->Height-15*5 );
        txtHelper.DrawTextLine( L"Rotate object: Left drag mouse\n"
                                L"Zoom In/Out: Mouse wheel\n"
                                L"Hide help: F1\n"
                                L"Quit: ESC" );
    }
    else
    {
        txtHelper.SetForegroundColor( D3DXCOLOR( 1.0f, 0.75f, 0.0f, 1.0f ) );
        txtHelper.DrawTextLine( L"Press F1 for help" );
    }

    txtHelper.End();
}


//--------------------------------------------------------------------------------------
// As a convenience, DXUT inspects the incoming windows messages for
// keystroke messages and decodes the message parameters to pass relevant keyboard
// messages to the application.  The framework does not remove the underlying keystroke 
// messages, which are still passed to the application's MsgProc callback.
//--------------------------------------------------------------------------------------
void CALLBACK KeyboardProc( UINT nChar, bool bKeyDown, bool bAltDown, void* pUserContext )
{
    if( bKeyDown )
    {
        switch( nChar )
        {
            case VK_F1: g_bShowHelp = !g_bShowHelp; break;
        }
    }
}


//--------------------------------------------------------------------------------------
// Handle messages to the application 
//--------------------------------------------------------------------------------------
LRESULT CALLBACK MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, 
                          bool* pbNoFurtherProcessing, void* pUserContext )
{
    // Always allow dialog resource manager calls to handle global messages
    // so GUI state is updated correctly
    *pbNoFurtherProcessing = g_DialogResourceManager.MsgProc( hWnd, uMsg, wParam, lParam );
    if( *pbNoFurtherProcessing )
        return 0;

    // Give the dialogs a chance to handle the message first
    *pbNoFurtherProcessing = g_SampleUI.MsgProc( hWnd, uMsg, wParam, lParam );
    if( *pbNoFurtherProcessing )
        return 0;

    // Pass all remaining windows messages to camera so it can respond to user input
    g_Camera.HandleMessages( hWnd, uMsg, wParam, lParam );

    return 0;
}


//--------------------------------------------------------------------------------------
// Handles the GUI events
//--------------------------------------------------------------------------------------
void CALLBACK OnGUIEvent( UINT nEvent, int nControlID, CDXUTControl* pControl, void* pUserContext )
{
    switch( nControlID )
    {
        case IDC_SCROLLLEFT:
        case IDC_SCROLLRIGHT:
        {
			if(g_vr)
			{
				if(nControlID == IDC_SCROLLLEFT)
				{
					if(g_vehicleIndex > 0)
						g_requestedVehicleIndex = g_vehicleIndex - 1;
				}else
				{
					if((g_vehicleIndex + 1) < g_vr->m_header.numMeshes)
						g_requestedVehicleIndex = g_vehicleIndex + 1;
				}

				if(g_requestedVehicleIndex != 0xFFFF)
				{
					WCHAR wsz[256];
					StringCchPrintf( wsz, 256, L"Vehicle Number: %u", g_requestedVehicleIndex );
					wsz[255] = L'\0';
					g_SampleUI.GetStatic( IDC_VEHICLE_NUMBER )->SetText( wsz );
				}
			}
            break;
        }

        case IDC_SHOW_ALL:
            g_nShowMode = 0;
			g_requestedVehicleIndex = g_vehicleIndex;
            break; 

        case IDC_SHOW_ONLY_TRI:
            g_nShowMode = 1;
			g_requestedVehicleIndex = g_vehicleIndex;
            break;

        case IDC_SHOW_ONLY_QUAD:
            g_nShowMode = 2;
			g_requestedVehicleIndex = g_vehicleIndex;
            break;
    }
}


//--------------------------------------------------------------------------------------
// Release resources created in the OnResetDevice callback here 
//--------------------------------------------------------------------------------------
void CALLBACK OnLostDevice( void* pUserContext )
{
    g_DialogResourceManager.OnLostDevice();

    if( g_pFont )
        g_pFont->OnLostDevice();

	SAFE_RELEASE(g_pTextSprite);
}


//--------------------------------------------------------------------------------------
// Release resources created in the OnCreateDevice callback here
//--------------------------------------------------------------------------------------
void CALLBACK OnDestroyDevice( void* pUserContext )
{
    g_DialogResourceManager.OnDestroyDevice();

	SAFE_RELEASE(g_vbuf);
	SAFE_RELEASE(g_texture);
    SAFE_RELEASE( g_pFont );
}

//--------------------------------------------------------------------------------------
// Initialize the app 
//--------------------------------------------------------------------------------------
void InitApp()
{
    // Initialize dialogs
    g_SampleUI.Init( &g_DialogResourceManager );

    g_SampleUI.SetCallback( OnGUIEvent ); int iY = 60;
	g_SampleUI.AddStatic( IDC_VEHICLE_NUMBER, L"Vehicle Number: 0", 0, iY, 160, 20 );
    g_SampleUI.AddButton( IDC_SCROLLLEFT, L"<<", 5, iY += 24, 70, 24 );
    g_SampleUI.AddButton( IDC_SCROLLRIGHT, L">>", 85, iY, 70, 24 );

	iY = 20;
    g_SampleUI.AddRadioButton( IDC_SHOW_ALL, 1, L"Show All", 200, iY += 24, 125, 22, true );
	g_SampleUI.AddRadioButton( IDC_SHOW_ONLY_TRI, 1, L"Show Tri", 200, iY += 24, 125, 22 );
    g_SampleUI.AddRadioButton( IDC_SHOW_ONLY_QUAD, 1, L"Show Quad", 200, iY += 24, 125, 22 );

	// load our vehicles
	g_vr = sw_vehrd_read("./data/PRIMVEH.OBJ");	assert(g_vr);
}


//--------------------------------------------------------------------------------------
// Initialize everything and go into a render loop
//--------------------------------------------------------------------------------------
INT WINAPI WinMain( HINSTANCE, HINSTANCE, LPSTR, int )
{
    // Enable run-time memory check for debug builds.
#if defined(DEBUG) | defined(_DEBUG)
    _CrtSetDbgFlag( _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF );
#endif

    // Set the callback functions
    DXUTSetCallbackDeviceCreated( OnCreateDevice );
    DXUTSetCallbackDeviceReset( OnResetDevice );
    DXUTSetCallbackDeviceLost( OnLostDevice );
    DXUTSetCallbackDeviceDestroyed( OnDestroyDevice );
    DXUTSetCallbackMsgProc( MsgProc );
    DXUTSetCallbackKeyboard( KeyboardProc );
    DXUTSetCallbackFrameRender( OnFrameRender );
    DXUTSetCallbackFrameMove( OnFrameMove );
   
    // TODO: Perform any application-level initialization here
	InitApp();

    // Initialize DXUT and create the desired Win32 window and Direct3D device for the application
    DXUTInit( true, true, true ); // Parse the command line, handle the default hotkeys, and show msgboxes
    DXUTSetCursorSettings( true, true ); // Show the cursor and clip it when in full screen
    DXUTCreateWindow( WND_NAME );
    DXUTCreateDevice( D3DADAPTER_DEFAULT, true, 640, 480, IsDeviceAcceptable, ModifyDeviceSettings );

    // Start the render loop
    DXUTMainLoop();

    
	// TODO: Perform any application-level cleanup here
	if(g_vr) delete g_vr;


    return DXUTGetExitCode();
}


